@@ -9,12 +9,15 @@ import re
 import textwrap
 import pluginlib
 import ConfigParser
+import getpass
 try:
     import subprocess
 except ImportError:
     from paste.script.util import subprocess24 as subprocess
 difflib = None
 
+from paste.script.util.logging_config import fileConfig
+
 class BadCommand(Exception):
 
     def __init__(self, message, exit_code=2):
@@ -22,7 +25,7 @@ class BadCommand(Exception):
         self.exit_code = exit_code
         Exception.__init__(self, message)
 
-class NoDefault:
+class NoDefault(object):
     pass
 
 dist = pkg_resources.get_distribution('PasteScript')
@@ -275,7 +278,7 @@ class Command(object):
                 return response[0].lower() == 'y'
             print 'Y or N please'
 
-    def challenge(self, prompt, default=NoDefault):
+    def challenge(self, prompt, default=NoDefault, should_echo=True):
         """
         Prompt the user for a variable.
         """
@@ -283,7 +286,11 @@ class Command(object):
             prompt += ' [%r]' % default
         prompt += ': '
         while 1:
-            response = raw_input(prompt).strip()
+            if should_echo:
+                prompt_method = raw_input
+            else:
+                prompt_method = getpass.getpass
+            response = prompt_method(prompt).strip()
             if not response:
                 if default is not NoDefault:
                     return default
@@ -390,7 +397,7 @@ class Command(object):
                 os.mkdir(dir)
             if (svn_add and
                 os.path.exists(os.path.join(os.path.dirname(dir), '.svn'))):
-                self.run_command('svn', 'add', dir)
+                self.svn_command('add', dir)
         else:
             if self.verbose > 1:
                 print "Directory already exists: %s" % self.shorten(dir)
@@ -413,7 +420,8 @@ class Command(object):
                 f.write(content)
                 f.close()
             if svn_add and os.path.exists(os.path.join(os.path.dirname(filename), '.svn')):
-                self.run_command('svn', 'add', filename)
+                self.svn_command('add', filename,
+                                 warn_returncode=True)
             return
         f = open(filename, 'rb')
         old_content = f.read()
@@ -484,7 +492,8 @@ class Command(object):
         for i in range(len(lines)):
             if regex.search(lines[i]):
                 # Found it!
-                if lines[i:] and len(lines[i:]) > 1 and lines[i+1].strip() == text.strip():
+                if (lines[i:] and len(lines[i:]) > 1 and
+                    ''.join(lines[i+1:]).strip().startswith(text.strip())):
                     # Already have it!
                     print 'Warning: line already found in %s (not inserting' % filename
                     print '  %s' % lines[i]
@@ -527,10 +536,14 @@ class Command(object):
         force_no_simulate:
             if true, run the command even if --simulate
         """
+        cmd = self.quote_first_command_arg(cmd)
         cwd = popdefault(kw, 'cwd', os.getcwd())
         capture_stderr = popdefault(kw, 'capture_stderr', False)
         expect_returncode = popdefault(kw, 'expect_returncode', False)
         force = popdefault(kw, 'force_no_simulate', False)
+        warn_returncode = popdefault(kw, 'warn_returncode', False)
+        if warn_returncode:
+            expect_returncode = True
         simulate = self.simulate
         if force:
             simulate = False
@@ -570,8 +583,45 @@ class Command(object):
             if stdout:
                 print 'Command output:'
                 print stdout
+        elif proc.returncode and warn_returncode:
+            print 'Warning: command failed (%s %s)' % (cmd, ' '.join(args))
+            print 'Exited with code %s' % proc.returncode
         return stdout
 
+    def quote_first_command_arg(self, arg):
+        """
+        There's a bug in Windows when running an executable that's
+        located inside a path with a space in it.  This method handles
+        that case, or on non-Windows systems or an executable with no
+        spaces, it just leaves well enough alone.
+        """
+        if (sys.platform != 'win32'
+            or ' ' not in arg):
+            # Problem does not apply:
+            return arg
+        try:
+            import win32api
+        except ImportError:
+            raise ValueError(
+                "The executable %r contains a space, and in order to "
+                "handle this issue you must have the win32api module "
+                "installed" % arg)
+        arg = win32api.GetShortPathName(arg)
+        return arg
+
+    _svn_failed = False
+
+    def svn_command(self, *args, **kw):
+        """
+        Run an svn command, but don't raise an exception if it fails.
+        """
+        try:
+            return self.run_command('svn', *args, **kw)
+        except OSError, e:
+            if not self._svn_failed:
+                print 'Unable to run svn command (%s); proceeding anyway' % e
+                self._svn_failed = True
+
     def write_file(self, filename, content, source=None,
                    binary=True, svn_add=True):
         """
@@ -614,7 +664,7 @@ class Command(object):
             if (not already_existed
                 and svn_add
                 and os.path.exists(os.path.join(os.path.dirname(filename), '.svn'))):
-                self.run_command('svn', 'add', filename)
+                self.svn_command('add', filename)
 
     def parse_vars(self, args):
         """
@@ -691,6 +741,16 @@ class Command(object):
         rest = [(' '*indent)+l for l in lines[1:]]
         return '\n'.join([first]+rest)
 
+    def logging_file_config(self, config_file):
+        """
+        Setup logging via the logging module's fileConfig function with the
+        specified ``config_file``, if applicable.
+        """
+        parser = ConfigParser.ConfigParser()
+        parser.read([config_file])
+        if parser.has_section('loggers'):
+            fileConfig(config_file)
+
 class NotFoundCommand(Command):
 
     def run(self, args):
